// Copyright (C) 2006 Michael Van Canneyt und Toolbox Verlag, Alle Rechte Vorbehalten
unit wmsession; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, HTTPDefs, fpHTTP, websession,fpWeb,
  IBConnection, sqldb, sqldbsession;

type

  { TSessionModule }

  TSessionModule = class(TFPWebModule)
    FPCDB: TIBConnection;
    SQLTransaction1: TSQLTransaction;
    procedure HaveSessionRequest(Sender: TObject; ARequest: TRequest;
      AResponse: TResponse; var Handled: Boolean);
    procedure SessionModuleCreate(Sender: TObject);
    procedure SessionModuleNewSession(Sender: TObject);
    procedure SessionStartedRequest(Sender: TObject; ARequest: TRequest;
      AResponse: TResponse; var Handled: Boolean);
    procedure TerminateRequest(Sender: TObject; ARequest: TRequest;
      AResponse: TResponse; var Handled: Boolean);
  private
    { private declarations }
  public
    { public declarations }
  end; 

var
  SessionModule: TSessionModule;

implementation
{ $define cgidebug}

{$ifdef cgidebug}
uses dbugintf;
{$endif cgidebug}

{ TSessionModule }

procedure TSessionModule.HaveSessionRequest(Sender: TObject;
  ARequest: TRequest; AResponse: TResponse; var Handled: Boolean);
  
Var
  C : TCookie;
  V : String;
  
begin
{$ifdef cgidebug}SendMethodEnter('TSessionModule.HaveSessionRequest');{$endif}
  With AResponse.Contents do
    begin
    Add('<HTML><TITLE>In Session</TITLE><BODY>');
    Add('<H1>In Session</H1>');
    Add('The session is still active');
{$ifdef cgidebug}SendDebug('Checking session');{$endif}
    If Session is TFPWebSession then
      begin
//      TFPWebSession(Session).Cached:=False;
{$ifdef cgidebug}SendDebug('Looking for cookie');{$endif}
      C:=AResponse.Cookies.FindCookie((Session as TFPWebSession).SessionCookie);
      If Assigned(C) then
        begin
{$ifdef cgidebug}SendDebug('Cookie found');{$endif}
        Add('Sending session cookie. Session Cookie is ');
        Add('called <B>'+C.Name+'</B> ');
        Add('and has value <B>'+C.Value+'</B>.');
        end;
      end;
    V:=Session.Variables['Value'];
    If (V<>'') then
      Add('<P>Stored session value: <B>'+V+'</B>.')
    else
      Add('<P>No values stored in session.');
    V:=ARequest.QueryFields.Values['Value'];
    If V<>'' then
      begin
      Add('<P>Storing new session value: <B>'+V+'</B>.');
      Session.Variables['Value']:=V;
      end;
    Add('</BODY></HTML>');
    end;
  Handled:=True;
{$ifdef cgidebug}SendMethodExit('TSessionModule.HaveSessionRequest');{$endif}
end;

procedure TSessionModule.SessionModuleCreate(Sender: TObject);

Var
  S :TSQLDBSession;

begin
  S:=TSQLDBSession.Create(Self);
  S.Connection:=FPCDB;
  Session:=S;
end;

procedure TSessionModule.SessionModuleNewSession(Sender: TObject);
begin
  Actions[1].Default:=False;
  Actions[0].Default:=True;
end;

procedure TSessionModule.SessionStartedRequest(Sender: TObject;
  ARequest: TRequest; AResponse: TResponse; var Handled: Boolean);
  
Var
  C : TCookie;
  
begin
{$ifdef cgidebug}SendMethodEnter('TSessionModule.SessionStarted');{$endif}
  With AResponse.Contents do
    begin
    Add('<HTML><TITLE>Session started</TITLE><BODY>');
    Add('<H1>Session started</H1>');
    Add('A session was started.');
    If Session is TFPWebSession then
      begin
      C:=AResponse.Cookies.FindCookie((Session as TFPWebSession).SessionCookie);
      If Assigned(C) then
        begin
        Add('Session Cookie is ');
        Add('called <B>'+C.Name+'</B> ');
        Add('and has value <B>'+C.Value+'</B>.');
        end
      else
        begin
        Add('Session cookie was not yet set.');
        end;
      end;
    Add('</BODY></HTML>');
    end;
{$ifdef cgidebug}SendDebug('Sending Content');{$endif}
  Handled:=True;
{$ifdef cgidebug}SendMethodExit('TSessionModule.SessionStarted');{$endif}
end;

procedure TSessionModule.TerminateRequest(Sender: TObject; ARequest: TRequest;
  AResponse: TResponse; var Handled: Boolean);
begin
{$ifdef cgidebug}SendMethodEnter('TSessionModule.TerminateRequest');{$endif}
  Session.Terminate;
  With AResponse.Contents do
    begin
    Add('<HTML><TITLE>Session terminated</TITLE><BODY>');
    Add('<H1>Session Terminated</H1>');
    Add('The session was terminated');
    Add('</BODY></HTML>');
    end;
  Handled:=True;
{$ifdef cgidebug}SendMethodExit('TSessionModule.TerminateRequest');{$endif}
end;

initialization
  {$I wmsession.lrs}

  RegisterHTTPModule('Session', TSessionModule);
end.

